#ifndef CONTROL_ROBOT_H_INCLUDED
#define CONTROL_ROBOT_H_INCLUDED

/**Todos los datos con los que se trabaja son:
Tiempo    = segundos
Velocidad = mm/s
Posicion  = mm
Angulo    = Grados*/
#define B         125        //Distancia de la rueda al centro del robot

/**variable para envio y recepcion de datos del socket*/
unsigned char g_buffer[51];
/**Manejador de socket*/
int g_sockfd;
/**Estructura que contiene todos los datos que hemos recibido*/
struct Datos_Control
{
	int nTipoRef; /**datos[0] = Tipo de trayectoria: 0 --> Rectangulo; 1 --> Circulo
	Si es rectangulo:
		datos[1-4] = LADO1
		datos[5-8] = LADO2*/
	int nLado1;
	int nLado2;
	/**Si es circulo:
		datos[1-4] = RADIO
		datos[5-8] = FRECUENCIA*/
    int nRadio;
    float fFrecuencia;
    /**Para alcanzar una posicion final*/
    int nPosFinalX;
    int nPosFinalY;
    /**datos[9-12] = VELOCIDAD DE REFERENCIA*/
	int nVelocidad;
	/**datos[13-16] = Periodo de muestreo*/
	float fTS;
	/**datos[17-20] = Posicion X inicial del robot en mm*/
	int nPosX;
	/**datos[21-24] = Posicion Y inicial del robot en mm*/
	int nPosY;
	/**datos[25-28] = Orientacin inicial del robot en grados*/
	float flOrientacion;
	/**datos[29] = Tipo de Control: 0 --> Punto descentralizado; 1 --> Persecucin pura
	Si es punto descentralizado:
		datos[30-33] = Punto E
		datos[34-37] = ganancia Kpx
		datos[38-41] = ganancia Kpy
		datos[42-45] = ganancia Kvx
		datos[46-49] = ganancia Kvy*/
	int nTipoControl;
    int nPuntoE;
    float fKPX;
    float fKPY;
    float fKVX;
    float fKVY;
	/**Si es persecucion pura
		datos[30-33] = ganancia k_rho
		datos[34-37] = ganancia k_alpha
		datos[38-41] = ganancia k_beta
		datos[42-45] = ganancia kp*/
    float k_rho;
	float k_alpha;
	float k_beta;
    float kp;
    /**raster, que es el tamao del cuadro de la camara cuando es trayectoria*/
    int nRaster;
    /**tamano del vector de referencias, ya que lo debo de saber a priori*/
    int nTamReferencias;
} g_DatosReferencia;

/**Vectores que componen las referencia*/
int *g_nPosicion_Ref_X;
int *g_nPosicion_Ref_Y;
int *g_nVelocidad_Ref_X;
int *g_nVelocidad_Ref_Y;
/**Tamao que tendra el vector, es decir, las iteraciones del bucle*/
int g_nTamano;
/**Componentes X e Y cuando es una trayectoria*/
int *g_nRefX;
int *g_nRefY;
/**Para saber si esta activada la parada de emergencia*/
int g_nParada;
/**Genera las posiciones y velocidades de referencia para hacer un rectangulo de lados nLado_1 y nLado_2*/
void Generar_Rectangulo(int nLado_1,int nLado_2);
/**Genera las posiciones y velocidades de referencia para hacer un circulo de radio nRadio y Frecuencia fFrecuencia*/
void Generar_Circulo(int nRadio,float fFrecuencia);
/**Genera las posiciones y velocidades de referencia para seguir una trayectoria generada con el MATLAB*/
void Generar_Trayectoria();
/**Genera las posiciones y velocidades de referencia para hacer una trayectoria en linea recta*/
void Generar_Trayectoria_Dos_Puntos();
/**Cuando se ejecuta la tarea coordinada para ir a por la pelota esta funcion genera la trayectoria
para hacer la mitad de un cuadrado*/
void Generar_Trayectoria_tres_PuntosIrPelota(int nPosInicialX,int nPosInicialY,int nPosFinalX,int nPosFinalY);
/**Cuando se ejecuta la tarea coordinada para ir a por la pelota esta funcion genera la trayectoria
para hacer la mitad de un cuadrado pero desde la pelota a la posicion inicial*/
void Generar_Trayectoria_tres_PuntosVolver(int nPosInicialX,int nPosInicialY,int nPosFinalX,int nPosFinalY);

/**Algoritmo de control de trayectoria por punto descentralizado*/
int Control_Trayectoria(int conexion);
/**Algoritmo de control de camino por persecucin pura*/
int Control_Camino(int conexion);
/**Funcin que rellena la estructura de 51 bytes recibidos para las ganancias de los algoritmos y todos los datos necesarios de
periodo, velocidad de referencia, etc*/
void RellenaEstructura();

/*********************************Sockets*****************************************/
/**Conecta los sockets y se pone a la espera */
int ConexionServidor();
/**Acepta nuevas peticiones*/
int NuevaPeticion();
/**Hilo para procesar el algoritmo de seguimiento de trayectoria o camino. es decir, se crea un nuevo hilo cada vez que recibimos
unja nueva peticin. Cuando acaba el procesamiento muere este y est preparado para recibir nuevas conexiones*/
void Procesa_Algoritmo(int conexion);
/**Hilo para procesar el algoritmo de recoger la pelota*/
void Procesa_Algoritmo_Pelota(int conexion);

/**Rellena estructura de posiciones para el algoritmo de seguimiento de trayectoria. Solo se utiliza parea el trabajo coordinado.
esta funcin es la que se encarga de rellenar las posiciones iniciales y finales*/
void RellenaPosiciones();

/**Las posiciones para el algoritmo de seguimiento son globales. Estas variables son las que rellena el algoritmo RellenaPosiciones*/
int g_nPosInicialX;
int g_nPosInicialY;
float g_flOrientaInicial;
int g_nPosFinalX;
int g_nPosFinalY;
float g_flOrientaFinal;
#endif // CONTROL_ROBOT_H_INCLUDED
