#include "IRCR.h"

#define TAM_BUFFER 50
#define BAUDRATE B57600
#define MODEMDEVICE "/dev/ttyS2"

#define BITCERO   0x00
#define BITUNO    0x01
#define BITDOS    0x02
#define BITTRES   0x04
#define BITCUATRO 0x08
#define BITCINCO  0x10
#define BITSEIS   0x20
#define BITSIETE  0x40

int g_iBytes; /**Para saber los bytes que escribo o leo del puerto*/
int g_hCom; /**Referencia al puerto creado*/
unsigned char g_buffer[TAM_BUFFER]; /**Donde almacenamos los bytes leidos o que se van a escribir, del puerto*/

/********** FUNCIONES DE LECTURA ESCRITURA CONEXIONES CON EL PERTO **********************************/

/**Funcion int ConectaPuerto(void). Esta funcin configuracionura todos los
parametros del puerto MODEMDEVICE, que es donde se encuentra ubicado el puerto serie del
Robot. Estos parametros son los Baudios de velocidad, bit de paridad,... Luego, mediante
el comando CreateFile creamos el puerto, como si de un fichero se tratara. Todo lo que
leamos o escribamos en el puerto se quedara almacenado en la variable g_buffer.*/
int ConectaPuerto()
{
   /* struct termios oldtio,newtio;

    g_hCom = open(MODEMDEVICE, O_RDWR | O_NOCTTY );
    if (g_hCom < 0)
        return 0;

    tcgetattr(g_hCom,&oldtio); /* save current port settings */

    /*bzero(&newtio, sizeof(newtio));
    newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
    newtio.c_iflag = IGNPAR;//IGNBRK;
    newtio.c_oflag = 0;

    /* set input mode (non-canonical, no echo,...) */
   /* newtio.c_lflag = 0;
    newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
    //Lo cambiamos a 50caracteres
  /*  newtio.c_cc[VMIN]     = 200;   /* blocking read until 5 chars received */

  /*  tcflush(g_hCom, TCIFLUSH);
    tcsetattr(g_hCom,TCSANOW,&newtio);
    return 1;*/

    struct termios oldkey,newkey;
    g_hCom = open(MODEMDEVICE, O_RDWR | O_NOCTTY); //set the user console port up
    tcgetattr(g_hCom,&oldkey); // save current port settings   //so commands are interpreted right for this program
    // set new port settings for non-canonical input processing  //must be NOCTTY
    newkey.c_cflag = BAUDRATE | CRTSCTS | CS8 | 0 | 0 | 0 | CLOCAL | CREAD;
    newkey.c_iflag = IGNPAR;
    newkey.c_oflag = 0;
    newkey.c_lflag = 0;       //ICANON;
    newkey.c_cc[VMIN]=1;
    newkey.c_cc[VTIME]=0;
    tcflush(g_hCom, TCIFLUSH);
    tcsetattr(g_hCom,TCSANOW,&newkey);

    return 1;

}

/**Funcin int DesconectaPuerto(). esta funcin desconecta el puerto de envio de comandos
del robot.*/
int DesconectaPuerto()
{
    if (!close(g_hCom) )
        return 0;
    return 1;
}

/**Funcin int LeePuerto(int bytes). Esta funcin lee del puerto los bytes indicados. Por ejemplo
si llamamos a esta funcion de la siguiente forma LeePuerto(3), lo que hace es leer 3 bytes
del puerto y deja el resultado en la variable g_buffer.*/
int LeePuerto(int bytes)
{
  if(!read(g_hCom, &g_buffer, bytes, &g_iBytes,NULL))
    return 0;
  return 1;
}

/**Funcion int EscribePuerto(int bytes). Esta funcin escribe en el puerto los datos
que tenemos en buffer. */
int EscribePuerto(int bytes)
{
  if(!write(g_hCom, g_buffer, bytes, &g_iBytes,NULL))
    return 0;
  return 1;
}

/**Funcion int ModoComando(Modo_Conexion mdCon). Esta funcin pone al robot a la espera
de comandos. Adems indicamos la veloclidad de transmision (57600 baudios) por si se ha
desconfigurado. tambin indicamos el modo de conexin que puede ser seguro o completo.
En el modo seguro el robot funciona teniendo en cuenta la caida de las ruedas y el desnivel de stas.
En el modo completo se tiene el todo el poder sobre el robot sin las restricciones del modo seguro*/
int ModoComando(Modo_Conexion mdCon)
{
    g_buffer[0] = 128;
    //apagado,pasivo,seguro,completo
    switch (mdCon) {
        case seguro:
            g_buffer[1] = 131;
            break;
        case completo:
            g_buffer[1] = 132;
            break;
        default: //No es un codigo valido
            return 0;
    }
    if (!EscribePuerto(2))
        return 0;
    //A 57600 Baudios
    g_buffer[0] = 129;
    g_buffer[1] = 10;
    if (EscribePuerto(2) )
    {
        //Ademas inicializo los contadores de distancia y angulo
        //short int nAux = 0;
        //if (Distancia(nAux) )
        //    return Angulo(nAux);
    }
    return 0;
}

/********** FUNCIONES DE LOS ACTUADORES *************************************************************/
/**Funcin int Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en
mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
int Velocidad(int nVelocidad, int nRadio)
{
    //Comprobamos que la velocidad est entre -500 mm y 500 mm
    if (nVelocidad < -500)
        nVelocidad = -500;
    if (nVelocidad > 500)
        nVelocidad = 500;
    //Comprobamos que el radio est entre -2000 y 2000 mm
    if (nRadio < -2000)
        nRadio = -2000;
    if (nRadio > 2000)
        nRadio = 2000;
    //[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]

    g_buffer[0] = 137; //Comando 137 es la velocidad
    g_buffer[1] = ((nVelocidad & 0xFF00) >> 8); // velocidad byte alto
    g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
    g_buffer[3] = ( (nRadio & 0xFF00) >> 8); //Radio byte alto
    g_buffer[4] = (nRadio & 0xFF); //Radio byte bajo
    return EscribePuerto(5);
}

/**Funcin int Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en
mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
int Velocidad_Recto(int nVelocidad)
{
    //Comprobamos que la velocidad est entre -500 mm y 500 mm
    if (nVelocidad < -500)
        nVelocidad = -500;
    if (nVelocidad > 500)
        nVelocidad = 500;

    //[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]
    g_buffer[0] = 137; //Comando 137 es la velocidad
    g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
    g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
    g_buffer[3] = 128;
    g_buffer[4] = 0;
    return EscribePuerto(5);
}

/**Funcin int Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en
mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
int Gira_Derecha(int nVelocidad)
{
    //Comprobamos que la velocidad est entre -500 mm y 500 mm
    if (nVelocidad < -500)
        nVelocidad = -500;
    if (nVelocidad > 500)
        nVelocidad = 500;

    //[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]
    g_buffer[0] = 137; //Comando 137 es la velocidad
    g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
    g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
    g_buffer[3] = 255;
    g_buffer[4] = 255;
    return EscribePuerto(5);
}

/**Funcin int Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en
mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
int Gira_Izquierda(int nVelocidad)
{
    //Comprobamos que la velocidad est entre -500 mm y 500 mm
    if (nVelocidad < -500)
        nVelocidad = -500;
    if (nVelocidad > 500)
        nVelocidad = 500;

    //[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]
    g_buffer[0] = 137; //Comando 137 es la velocidad
    g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
    g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
    g_buffer[3] = 0;
    g_buffer[4] = 1;
    return EscribePuerto(5);
}

/**Funcin int Velocidad_Independiente(int nRueda_Izq, int nRueda_Der). Con esta funcin indicamos
la velocidad en mm/s para la rueda izquierda y la rueda derecha de forma independiente.*/
int Velocidad_Independiente(int nRueda_Izq, int nRueda_Der)
{
    //Comprobamos que la velocidad est entre -500 mm y 500 mm
    if (nRueda_Izq < -500)
        nRueda_Izq = -500;
    if (nRueda_Izq > 500)
        nRueda_Izq = 500;

    if (nRueda_Der < -500)
        nRueda_Der = -500;
    if (nRueda_Der > 500)
        nRueda_Der = 500;
    //[145] [Right velocity high byte][Right velocity low byte] [Left velocity high byte][Left velocity low byte]
    g_buffer[0] = 145; //Comando 145 es la velocidad de cada rueda
    g_buffer[1] = ( (nRueda_Der & 0xFF00) >> 8); // velocidad der byte alto
    g_buffer[2] = (nRueda_Der & 0xFF); // Velocidad der byte bajo
    g_buffer[3] = ( (nRueda_Izq & 0xFF00) >> 8); //velocidad izq byte alto
    g_buffer[4] = (nRueda_Izq & 0xFF); //velocidad izq byte bajo
    return EscribePuerto(5);
}

/**Funcin int Actuadores::Salidas_Digitales(int nSalida1,int nSalida2,int nSalida3). Con esta
funcin ponemos las tres salidas digitales a nivel bajo o alto (0 o 1, respectivamente).
El 0 manda 0 voltios y el 1 logico envia 5 voltios. */
int Salidas_Digitales(int nSalida1,int nSalida2,int nSalida3)
{
    int nValor = 0;

    if (nSalida1 != 0 && nSalida1 != 1)
        nSalida1 = 0;
    if (nSalida2 != 0 && nSalida2 != 1)
        nSalida2 = 0;
    if (nSalida3 != 0 && nSalida3 != 1)
        nSalida3 = 0;
    //[147] [Output Bits]
    //Output bits = 0 0 0 0 0 nSalida3 nSalida2 nSalida 1 --> 1 BYTE
    //nSalida1 = Pin 19
    //nSalida2 = Pin 7
    //nSalida3 = Pin 20
    if (nSalida1 == 1)
        nValor = 1; //2 elevado a 0
    if (nSalida2 == 1)
        nValor += 2;//2 elevado a 1
    if (nSalida3 == 1)
        nValor += 4;//2 elevado a 2
    g_buffer[0] = 145; //Comando 145 es la velocidad de cada rueda
    g_buffer[1] = nValor; // Bits de salidas digitales
    return EscribePuerto(2);
}

/********************FUNCIONES DE LAS ENTRADAS ANALOGICAS, DIGITALES, SENSORES,...******************/
/**int Parachoque_Nivel_Ruedas(int*bParachoque,int*bRueda). Esta funcin nos
devuelve si el parachoques del robot est chocando o alguna de las ruedas est en el aire (suspendida).
El valor se devuelve en los argumentos pasados como referencia.*/
int Parachoque_Nivel_Ruedas(int* bParachoque,int* bRueda)
{

    char bit_0,bit_1,bit_2,bit_3,bit_4;
    int nObtenido;

    g_buffer[0] = 142; //Comando 142 pide dato del sensor indicado
    g_buffer[1] = 7; // Estado de las ruedas y parachoques
    if (!EscribePuerto(2) )
        return 0;
    //Recojo la informacin, Es 1 byte 0 0 0 R_Castor R_Izq R_Der Golpe_Izq Golpe_Der
    if (!LeePuerto(1) )
        return 0;
    //En g_buffer tengo el byte de informacion. Extraigo los componentes
    nObtenido = g_buffer[0];
    bit_0 = (nObtenido & BITUNO);
    bit_1 = (nObtenido & BITDOS) >> 1;
    bit_2 = (nObtenido & BITTRES) >> 2;
    bit_3 = (nObtenido & BITCUATRO) >> 3;
    bit_4 = (nObtenido & BITCINCO) >> 4;
    //Bits 0 y 1 me dicen si hay golpe
    if (bit_0 == 1 || bit_1 == 1)
        *bParachoque = 1;
    else
        *bParachoque = 0;
    //Bits 2, 3 y 4 me dicen si hay caida de ruedas
    if (bit_2 == 1 || bit_3 == 1 || bit_4 == 1)
        *bRueda = 1;
    else
        *bRueda = 0;
    return 1;
}

/**int Parachoque_Nivel_Ruedas(int*bParachoque,int*bRueda). Esta funcin nos
devuelve si el parachoques del robot est chocando o alguna de las ruedas est en el aire (suspendida).
El valor se devuelve en los argumentos pasados como referencia.*/
int Sensores_Choque(int* nChoque_Izq,int* nChoque_Der)
{

    char bit_0,bit_1;
    int nObtenido;

    g_buffer[0] = 142; //Comando 142 pide dato del sensor indicado
    g_buffer[1] = 7; // Estado de las ruedas y parachoques
    if (!EscribePuerto(2) )
        return 0;
    //Recojo la informacin, Es 1 byte 0 0 0 R_Castor R_Izq R_Der Golpe_Izq Golpe_Der
    if (!LeePuerto(1) )
        return 0;
    //En g_buffer tengo el byte de informacion. Extraigo los componentes
    nObtenido = g_buffer[0];
    bit_0 = (nObtenido & BITUNO);
    bit_1 = (nObtenido & BITDOS) >> 1;
    *nChoque_Der = bit_0;
    *nChoque_Izq = bit_1;
    return 1;
}

/**int Muro(int*bMuro). Esta funcin devuelve si se detecta muro o no. Este valor
se devuelve en el parmetro pasado por referencia.*/
int Muro(int* bMuro)
{
    int nObtenido;
    g_buffer[0] = 142; //Comando 142 pide dato del sensor indicado
    g_buffer[1] = 8; // Estado de las ruedas y parachoques
    if (!EscribePuerto(2) )
        return 0;
    //Recojo la informacin, Es 1 byte 0 0 0 0 0 0 0 Muro
    if (!LeePuerto(1) )
        return 0;
    //En g_buffer tengo el byte de informacion. Extraigo los componentes
    nObtenido = g_buffer[0];
    if (nObtenido == 1)
        *bMuro = 1;
    else
        *bMuro = 0;
    return 1;
}

/**Funcin int Desnivel(int*bDesnivel). Esta funcin mira si hay algn desnivel en
todo el conjunto de las ruedas. Para consultar las 4 posibilidades (desnivel izquierdo,
izquierdo-frontal, derecho y derecho-frontal) mandamos un paquete encapsulado de tal forma
que en el mismo paquete obtenemos el estado de los 4 sensores*/
int Desnivel(int* bDesnivel)
{
    g_buffer[0] = 149; //Comando 149 pide dato de los sensores en forma lista
    g_buffer[1] = 4; //Cuantos sensores voy a pedir
    g_buffer[2] = 9;  // Desnivel izquierdo
    g_buffer[3] = 10; // Desnivel izquierdo-frontal
    g_buffer[4] = 11; // Desnivel derecho-frontal
    g_buffer[5] = 12; // Desnivel derecho
    if (!EscribePuerto(6) )
        return 0;
    //Obtendre 4 bytes. Del 9, 10, 11 y 12, ya que lo devuelve en el mismo orden
    //Para cada byte o devuelve 0 o 1
    if (!LeePuerto(4) )
        return 0;
    if (g_buffer[0] == 1 || g_buffer[1] == 1 || g_buffer[2] == 1 || g_buffer[3] == 1)
        *bDesnivel = 1;
    else
        *bDesnivel = 0;
    return 1;
}

/**Funcin int Muro_Virtual(int*bMuroV). Con esta funcin se detecta si existe o
no el muro virtual, es decir si existe es que no puede pasar el robot.*/
int Muro_Virtual(int* bMuroV)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 13;  //Sensor del muro virtual
    if (!EscribePuerto(2) )
        return 0;
    //Obtendre 1 bytes que vale 0 1 si no se detecta o si el muro, respectivamente
    if (!LeePuerto(1) )
        return 0;
    if (g_buffer[0] == 1)
        *bMuroV = 1;
    else
        *bMuroV = 0;
    return 1;
}

/**Funcin int Infrarojos(int*nEstado). Con esta funcin se el valor del
sensor infrarrojo del robot. Para todos los casos hay que comprar dispositivos adicionales
para poder usar este mtodo. En cualquier caso en el parmetro nestado se pone el valor
obtenido, cuyo valor se explica a continuacin:
Remote Control:
    129 Left
    130 Forward
    131 Right
    132 Spot
    133 Max
    134 Small
    135 Medium
    136 Large / Clean
    137 PAUSE
    138 Power
    139 arc-forward-left
    140 arc-forward-right
    141 drive-stop.
Scheduling Remote
    142 Send All
    143 Seek Dock
Home Base
    240 Reserved
    248 Red Buoy
    244 Green Buoy
    242 Force Field
    252 Red Buoy and Green Buoy
    250 Red Buoy and Force Field
    246 Green Buoy and Force Field
    254 Red Buoy, Green Buoy and Force Field
    255 No received byte. */
int Infrarojos(short int* nEstado)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 17;  //Sensor infrarrojo
    if (!EscribePuerto(2) )
        return 0;
    //Obtendre 1 bytes que vale 0 1 si no se detecta o si el muro, respectivamente
    if (!LeePuerto(1) )
        return 0;
    *nEstado = g_buffer[0];
    return 1;
}

/**Con esta funcin obtenemos el estado de los botones de play y avance. Si estan pulsados
en los parametros por referencia se devuelve 1 y sino 0*/
int Botones(int* bPlay,int* bAvance)
{
    int bit_0,bit_2, nObtenido;

    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 18;  //Sensor botones
    if (!EscribePuerto(2) )
        return 0;
    //Obtendre 1 bytes de la siguiente forma: 0 0 0 0 0 avance 0 play
    if (!LeePuerto(1) )
        return 0;
    nObtenido = g_buffer[0];
    bit_0 = (nObtenido & BITUNO);
    bit_2 = (nObtenido & BITTRES) >> 2;
    if (bit_0 == 1)
        *bPlay = 1;
    else
        *bPlay = 0;
    if (bit_2 == 1)
        *bAvance = 1;
    else
        *bAvance = 0;
    return 1;
}

/**Esta funcin obitiene la distancia del robot en mm. Para ello se obtiene la distancia de las
dos ruedas y se divide entre dos. Cada vez que se llama a esta funcin, ese contador se
incializa a 0 para la prxima vez que se llama. Si el valor es negativo significa que el
robot ha ido hacia atras*/
int Distancia(short int* nDistancia)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 19;  //Sensor encoders de las ruedas
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nDistancia = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}

/**Esta funcin obitiene el angulo de las ruedas del robot en grados. Cada vez que se llama
a esta funcin, ese contador se incializa a 0 para la prxima vez que se llama.*/
int Angulo(short int* nAngulo)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 20;  //Sensor encoders de las ruedas para el angulo
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nAngulo = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene como se encuentra la bateria. Cargando, espera,...
0 Not charging
1 Reconditioning Charging
2 Full Charging
3 Trickle Charging
4 Waiting
5 Charging Fault Condition*/
int Estado_Bateria(short int* nEstado)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 21;  //Estado de la bateria
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(1) )
        return 0;
    *nEstado = g_buffer[0];
    return 1;
}
/**Obtiene el voltaje de la bateria en mV*/
int Voltaje_Bateria(short int* nVoltaje)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 22;  //Voltaje de la bateria
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nVoltaje = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la corriente de la bateria en mA*/
int Corriente_Bateria(short int* nMiliamperios)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 23;  //Miliamperios de la bateria
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nMiliamperios = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la temperatura de la bateria en grados Celsius*/
int Temperatura_Bateria(short int*nTemperatura)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 24;  //Temperatura de la bateria
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(1) )
        return 0;
    *nTemperatura = g_buffer[0];
    return 1;
}
/**Obtiene la carga que le queda a la bateria en mA hora*/
int Carga_Bateria(short int* nCarga)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 25;  //Carga de la bateria que le queda
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nCarga = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la capacidad de la bateria en mA hora*/
int Capacidad_Bateria(short int* nCapacidad)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 26;  //Capacidad de la bateria en mAHora
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nCapacidad = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de muro*/
int Senal_Muro(short int* nSMuro)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 27;  //Intensidad de la seal del muro
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSMuro = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de desnivel izquierdo*/
int Senal_Desnivel_Izquierdo(short int* nSIzq)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 20;  //Intensidad del sensor de desnivel izquierdo
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSIzq = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de desnivel izquierdo-frontal*/
int Senal_Desnivel_Izquierdo_Fr(short int* nSIzq_Fr)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 29;  //Intensidad del sensor de desnivel izquierdo frontal
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSIzq_Fr = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de desnivel derecho-frontal*/
int Senal_Desnivel_Derecho_Fr(short int* nSDer_Fr)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 30;  //Intensidad del sensor de desnivel derecho frontal
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSDer_Fr = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de desnivel derecho*/
int Senal_Desnivel_Derecho(short int* nSDer)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 31;  //Intensidad del sensor de desnivel derecho
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSDer = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene la intensidad del sensor de los 4 desniveles*/
int Senal_Desniveles(short int* nSIzq,short int* nSIzq_Fr, short int* nSDer_Fr, short int* nSDer)
{
    g_buffer[0] = 149; //Comando 149 pide dato de los sensores en forma lista
    g_buffer[1] = 4;   //Cuantos sensores voy a pedir
    g_buffer[2] = 28;  // intensidad Desnivel izquierdo
    g_buffer[3] = 29;  // intensidad Desnivel izquierdo-frontal
    g_buffer[4] = 30;  // intensidad Desnivel derecho-frontal
    g_buffer[5] = 31;  // intensidad Desnivel derecho
    if (!EscribePuerto(6) )
        return 0;
    if (!LeePuerto(8) )
        return 0;
    //Los 2 bytes vienen alreves
    *nSIzq = g_buffer[1] + (g_buffer[0] << 8);
    //Los 2 bytes vienen alreves
    *nSIzq_Fr = g_buffer[3] + (g_buffer[2] << 8);
    //Los 2 bytes vienen alreves
    *nSDer_Fr = g_buffer[5] + (g_buffer[4] << 8);
    //Los 2 bytes vienen alreves
    *nSDer = g_buffer[7] + (g_buffer[6] << 8);
    return 1;

}
/**Obtiene el valor de las 4 entradas digitales. esta entrada se corresponden
con los pines 6,18, 5 y 17 para las entradas 3, 2, 1 y 0, respectivamente.*/
int Entradas_Digitales(int* bEntrada0, int* bEntrada1, int* bEntrada2, int* bEntrada3)
{
    int nEntrada0,nEntrada1,nEntrada2,nEntrada3,nObtenido;

    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 32;  //Entradas digitales
    if (!EscribePuerto(2) )
        return 0;
    //Obtendre 1 bytes de la siguiente forma 0 0 0 baudios E3 E2 E1 E0
    if (!LeePuerto(1) )
        return 0;
    //Los 2 bytes vienen alreves
    nObtenido = g_buffer[0];
    nEntrada0 = (nObtenido & BITUNO);
    nEntrada1 = (nObtenido & BITDOS) >> 1;
    nEntrada2 = (nObtenido & BITTRES) >> 2;
    nEntrada3 = (nObtenido & BITCUATRO) >> 3;
    if (nEntrada0 == 1)
        *bEntrada0 = 1;
    else
        *bEntrada0 = 0;

    if (nEntrada1 == 1)
        *bEntrada1 = 1;
    else
        *bEntrada1 = 0;

    if (nEntrada2 == 1)
        *bEntrada2 = 1;
    else
        *bEntrada2 = 0;

    if (nEntrada3 == 1)
        *bEntrada3 = 1;
    else
        *bEntrada3 = 0;

    return 1;
}
/**Obtiene el valor de la entradas analogica. El valor que obtenemos va
de 0 a 1023 (0 = 0V y 1023 = 5V). El pin de la entrada es el 4*/
int Entrada_Analogica(short int* nEntrada)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 33;  //Entrada analogica
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nEntrada = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene las fuentes de carga disponibles*/
int Fuente_Carga(int* bHome,int* bInterno)
{
    int nHome,nInterno,nObtenido;

    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 34;  //Fuentes de carga disponible
    if (!EscribePuerto(2) )
        return 0;
    //Obtendre 1 bytes de la siguiente forma 0 0 0 0 0 0 HOME INTERNO
    if (!LeePuerto(1) )
        return 0;
    //Los 2 bytes vienen alreves
    nObtenido = g_buffer[0];
    nInterno = (nObtenido & BITUNO);
    nHome = (nObtenido & BITDOS) >> 1;
    if (nInterno == 1)
        *bInterno = 1;
    else
        *bInterno = 0;

    if (nHome == 1)
        *bHome = 1;
    else
        *bHome = 0;

    return 1;
}
/**Obtiene el modo OI en el que esta trabajando:
0 Off
1 Passive
2 Safe
3 Full*/
int Modo_OI(short int* nModo)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 35;  //Modo OI
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(1) )
        return 0;
    //Los 2 bytes vienen alreves
    *nModo = g_buffer[0];
    return 1;
}
/**Obtiene el ultimo valor de velocidad que se pidio. En teoria debe de
coincidir con la velocidad del robot*/
int Velocidad_Solicitada(short int* nVelocidad)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 39;  //Velocidad del robot que se solicito
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nVelocidad = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene el ultimo valor de radio solicitado. En teoria debe de coincidir
con el radio de las ruedas del robot*/
int Radio_Solicitado(short int* nRadio)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 40;  //Radio de las ruedas
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nRadio = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene el ultimo valor de velocidad de la rueda derecha que se pidio. */
int Velocidad_Solic_Derecha(short int* nVelocidad_Der)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 41;  //Velocidad R derecha
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nVelocidad_Der = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}
/**Obtiene el ultimo valor de velocidad de la rueda izquierda que se pidio. */
int Velocidad_Solic_Izquierda(short int* nVelocidad_Izq)
{
    g_buffer[0] = 142; //Comando 142 pide dato del sensor
    g_buffer[1] = 42;  //Velocidad R izquierda
    if (!EscribePuerto(2) )
        return 0;
    if (!LeePuerto(2) )
        return 0;
    //Los 2 bytes vienen alreves
    *nVelocidad_Izq = g_buffer[1] + (g_buffer[0] << 8);
    return 1;
}

/***************************FUNCIONES DE EVENTOS****************************************************/
/**Con esta funcin espera el tiempo indicado en nTiempo en decimas de segundo. El tiempo va
desde 0 a 255 decimas de segundo*/
int Esperar_Tiempo(short int nTiempo)
{
    if (nTiempo < 0)
        nTiempo = 0;
    if (nTiempo > 255)
        nTiempo = 255;
    g_buffer[0] = 155; //Comando 155 esperar tiempo
    g_buffer[1] = nTiempo; //Tiempo indicado en decimas de segundo
    return EscribePuerto(2);
}
/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
la distancia indicada en mm. Es decir, hasta que no se recorra nDistancia mm el robot
no ejecuta la siguiente instruccion. Adems, la variable interna de la distancia se
inicializa a 0. La distancia puede ser desde -32767 mm a 32768 mm*/
int Esperar_Distancia(short int nDistancia)
{
    if (nDistancia < -32767)
        nDistancia = -32767;
    if (nDistancia > 32768)
        nDistancia = 32768;
    g_buffer[0] = 156; //Comando 156 esperar distancia
    g_buffer[1] = ( (nDistancia & 0xFF00) >> 8); // Distancia byte alto
    g_buffer[2] = (nDistancia & 0xFF); // Distancia byte bajo
    return EscribePuerto(3);
}
/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
el angulo indicado en grados. Adems, la variable interna del ngulo se inicializa a 0. El valor
del angulo va desde -32767 a 32768 grados.*/
int Esperar_Angulo(short int nAngulo)
{
    if (nAngulo < -32767)
        nAngulo = -32767;
    if (nAngulo > 32768)
        nAngulo = 32768;
    g_buffer[0] = 157; //Comando 157 esperar angulo
    g_buffer[1] = ( (nAngulo & 0xFF00) >> 8); // Angulo byte alto
    g_buffer[2] = (nAngulo & 0xFF); // Angulo byte bajo
    return EscribePuerto(3);
}
/**Con esta funcin esperamos a que ocurra el evento que se especifica en el parametro.
Este parmetro es del tipo Evento y segn el nombre de la enumeracin ya indica en si
que evento es el que se espera*/
int Esperar_Evento(Evento evento)
{
    g_buffer[0] = 158; //Comando 158 esperar evento
    g_buffer[1] = evento; // que tipo de evento es el que queremos esperar
    return EscribePuerto(2);
}

