#ifndef IRCR_H_INCLUDED
#define IRCR_H_INCLUDED

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>

#if defined (__cplusplus)
extern "C" {
#endif

/********** FUNCIONES DE LECTURA ESCRITURA CONEXIONES CON EL PERTO **********************************/
/**Definicion de estructuras y variables*/
typedef enum {seguro,completo} Modo_Conexion;
/**Conecta con el puerto serie especificado*/
int ConectaPuerto();
/**Desconecta el puerto*/
int DesconectaPuerto();
/**Lee del puerto abierto tantos bytes como se indiquen y los deja en g_buffer*/
int LeePuerto(int bytes);
/**Escribe en el puerto indicado el contenido de g_buffer*/
int EscribePuerto(int bytes);
/**Pone al robot en modo OI para poder enviar y recibir informacion por el puerto serie*/
int ModoComando(Modo_Conexion mdCon);

/********** FUNCIONES DE LOS ACTUADORES *************************************************************/
/**Pone las dos ruedas del robot a girar a la velocidad indicada y con el ngulo de giro indicado*/
int Velocidad(int nVelocidad, int nRadio);
/**Velocidad para que vaya recto*/
int Velocidad_Recto(int nVelocidad);
/**Velocidad para que gire sentido horario*/
int Gira_Derecha(int nVelocidad);
/**Velocidad para que gire sentido antihorario*/
int Gira_Izquierda(int nVelocidad);
/**Cambia la velocidad de las ruedas de forma independiente*/
int Velocidad_Independiente(int nRueda_Izq, int nRueda_Der);
/**Cambia el valor de las tres salidas digitales del robot (pines 19, 7 y 20 del cargo bay conector*/
int Salidas_Digitales(int nSalida1, int nSalida2, int nSalida3);

/*********************FUNCIONES DE LAS ENTRADAS ANALOGICAS, DIGITALES, SENSORES,...******************/
/**Obtiene el estado del parachoques y de la caida de las 3 ruedas*/
int Parachoque_Nivel_Ruedas(int* bParachoque,int* bRueda);
/**Obtiene el estados de los dos sensores de choque*/
int Sensores_Choque(int* nChoque_Izq,int* nChoque_Der);
/**Obtiene el estado de muro para saber si hay obstaculo o no*/
int Muro(int* bMuro);
/**Obtiene si alguna de las ruedas esta desniveladas, que son desnivel izquierdo,
izquierdo frontal, derecho y derecho-frontal*/
int Desnivel(int* bDesnivel);
/**Obtiene si hay choque con el muro virtual. De momento en desuso*/
int Muro_Virtual(int* bMuroV);
/**Obtiene el valor del infrarojo. De momento en desuso*/
int Infrarojos(short int* nEstado);
/**Obtiene el estado de los botones de play y avance*/
int Botones(int* bPlay,int* bAvance);
/**Obtiene la distancia en mm que ha recorrido el robot. Cada vez que se llama a esta
funcin se inicializa el contador a 0. Es decir, si la llamo una vez y me devuelve
5 mm, ese miusmo contador se vuelve a inicializar a 0 desde ese momento.*/
int Distancia(short int* nDistancia);
/**Obtiene el ngulo engrados de las ruedas del robot. Cada vez que se llama a esta
funcin se inicializa el contador a 0.*/
int Angulo(short int* nAngulo);
/**Obtiene como se encuentra la bateria. Cargando, espera,...*/
int Estado_Bateria(short int* nEstado);
/**Obtiene el voltaje de la bateria en mV*/
int Voltaje_Bateria(short int* nVoltaje);
/**Obtiene la corriente de la bateria en mA*/
int Corriente_Bateria(short int* nMiliamperios);
/**Obtiene la temperatura de la bateria en grados Celsius*/
int Temperatura_Bateria(short int* nTemperatura);
/**Obtiene la carga que le queda a la bateria en mA hora*/
int Carga_Bateria(short int* nCarga);
/**Obtiene la capacidad de la bateria en mA hora*/
int Capacidad_Bateria(short int* nCapacidad);
/**Obtiene la intensidad del sensor de muro*/
int Senal_Muro(short int* nSMuro);
/**Obtiene la intensidad del sensor de desnivel izquierdo*/
int Senal_Desnivel_Izquierdo(short int* nSIzq);
/**Obtiene la intensidad del sensor de desnivel izquierdo-frontal*/
int Senal_Desnivel_Izquierdo_Fr(short int* nSIzq_Fr);
/**Obtiene la intensidad del sensor de desnivel derecho-frontal*/
int Senal_Desnivel_Derecho_Fr(short int* nSDer_Fr);
/**Obtiene la intensidad del sensor de desnivel derecho*/
int Senal_Desnivel_Derecho(short int* nSDer);
/**Obtiene la intensidad del sensor de los 4 desniveles*/
int Senal_Desniveles(short int* nSIzq,short int* nSIzq_Fr,short int* nSDer_Fr,short int* nSDer);
/**Obtiene el valor de las 4 entradas digitales*/
int Entradas_Digitales(int* bEntrada0, int* bEntrada1, int* bEntrada2, int* bEntrada3);
/**Obtiene el valor de la entradas analogica*/
int Entrada_Analogica(short int* nEntrada);
/**Obtiene las fuentes de carga disponibles*/
int Fuente_Carga(int* bHome,int* bInterno);
/**Obtiene el modo OI en el que esta trabajando */
int Modo_OI(short int* nModo);
/**Obtiene el ultimo valor de velocidad que se pidio. En teoria debe de
coincidir con la velocidad del robot*/
int Velocidad_Solicitada(short int* nVelocidad);
/**Obtiene el ultimo valor de radio solicitado. En teoria debe de coincidir
con el radio de las ruedas del robot*/
int Radio_Solicitado(short int* nRadio);
/**Obtiene el ultimo valor de velocidad de la rueda derecha que se pidio. */
int Velocidad_Solic_Derecha(short int* nVelocidad_Der);
/**Obtiene el ultimo valor de velocidad de la rueda izquierda que se pidio. */
int Velocidad_Solic_Izquierda(short int* nVelocidad_Izq);

/***************************FUNCIONES DE EVENTOS****************************************************/
/**Definicion de estructura para el evento de espera*/
typedef enum {Rueda_Caida = 1,Rueda_Caida_FR = 2,Rueda_Caida_Izq = 3,
Rueda_Caida_Der = 4,Parachoque = 5,Parachoque_Izq = 6,Parachoque_Der = 7,Muro_Virtual_e = 8,
Muro_e = 9,Desnivel_e = 10,Desnivel_Izq_e = 11,Desnivel_Izq_Fr_e = 12,Desnivel_Der_Fr_e = 13,
Desnivel_Der_e = 14, Home_Base = 15, Boton_Avance = 16, Boton_Play = 17, Entrada_Digital_0 = 18,
Entrada_Digital_1 = 19, Entrada_Digital_2 = 20, Entrada_Digital_3 = 21, Modo_OI_Pasivo = 22,
NO_Rueda_Caida = 255,NO_Rueda_Caida_FR = 254,NO_Rueda_Caida_Izq = 253,
NO_Rueda_Caida_Der = 252,NO_Parachoque = 251,NO_Parachoque_Izq = 250,NO_Parachoque_Der = 249,NO_Muro_Virtual = 248,
NO_Muro = 247,NO_Desnivel = 246,NO_Desnivel_Izq = 245,NO_Desnivel_Izq_Fr = 244,NO_Desnivel_Der_Fr = 243,
NO_Desnivel_Der = 242, NO_Home_Base = 241, NO_Boton_Avance = 240, NO_Boton_Play = 239, NO_Entrada_Digital_0 = 238,
NO_Entrada_Digital_1 = 237, NO_Entrada_Digital_2 = 236, NO_Entrada_Digital_3 = 235, NO_Modo_OI_Pasivo = 234} Evento;

/**Con esta funcin espera el tiempo indicado en nTiempo en decimas de segundo*/
int Esperar_Tiempo(short int nTiempo);
/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
la distancia indicada en mm. Es decir, hasta que no se recorra nDistancia mm el robot
no ejecuta la siguiente instruccion.*/
int Esperar_Distancia(short int nDistancia);
/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
el angulo indicado en grados.*/
int Esperar_Angulo(short int nAngulo);
/**Con esta funcin esperamos a que ocurra el evento que se especifica en el parametro.
Este parmetro es del tipo Evento y segn el nombre de la enumeracin ya indica en si
que evento es el que se espera*/
int Esperar_Evento(Evento evento);

#if defined (__cplusplus)
}
#endif

#endif // IRCR_H_INCLUDED
