#include "IRCR_API.h"
#include <stdlib.h>
#include <stdio.h>
#include <stdexcept>

DCB g_dcb; /**Para crear el puerto*/		
DWORD g_iBytes; /**Para saber los bytes que escribo o leo del puerto*/
HANDLE g_hCom; /**Referencia al puerto creado*/
unsigned char g_buffer[TAM_BUFFER]; /**Donde almacenamos los bytes leidos o que se van a escribir, del puerto*/
HANDLE  g_mutex; /**Semaforo para compartir el recurso, es decir, el puerto*/

using namespace std;

namespace IRCR
{
	/**Esta clase es utilizada para conexin, desconexin y	tratamiento del puerto, as como 
	poner el robot en modo OI (preparado para enviar y recibir instrucciones por el puerto serie 
	virtual*/
	Conexion::Conexion(){} 

	/**Destructor Conexion::~Conexion()*/
	Conexion::~Conexion(){} 

	/**Funcion bool Conexion::ConectaPuerto(char *puerto). Esta funcin configuracionura todos los 
	parametros del puerto "puerto", que es donde se encuentra ubicado el puerto serie del 
	Robot. Estos parametros son los Baudios de velocidad, bit de paridad,... Luego, mediante 
	el comando CreateFile creamos el puerto, como si de un fichero se tratara. Todo lo que 
	leamos o escribamos en el puerto se quedara almacenado en la variable g_buffer.*/
	bool Conexion::ConectaPuerto(char* puerto)
	{
		  //Convettirmo a wchar_t *
		  size_t origsize = strlen(puerto) + 1;
		  const size_t newsize = 100;
		  size_t convertedChars = 0;
		  wchar_t wcstring[newsize];
		  mbstowcs_s(&convertedChars, wcstring, origsize, puerto, _TRUNCATE);
		  //Creamos el puerto
		  g_hCom = CreateFile(wcstring, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
	  
		  // Comprobamos resultado de apertura
		  if (g_hCom == INVALID_HANDLE_VALUE) 
			  return false;
	  
		  //PARAMETROS DEL PUERTO
		  if (GetCommState(g_hCom, &g_dcb)==0)
			  return false;
		  
		  //Establecemos velocidad a BAUDIOS
		  g_dcb.BaudRate = BAUDIOS;
		  g_dcb.fParity = FALSE;
		  g_dcb.fOutxCtsFlow = FALSE;
		  g_dcb.fOutxDsrFlow = FALSE;
		  g_dcb.fDtrControl = DTR_CONTROL_DISABLE;
		  g_dcb.fDsrSensitivity = FALSE;
		  g_dcb.fOutX = FALSE;
		  g_dcb.fInX = FALSE;
		  g_dcb.fRtsControl = RTS_CONTROL_DISABLE;
		  g_dcb.fAbortOnError = FALSE;
		  //Un byte
		  g_dcb.ByteSize = 8;
		  //Sin paridad
		  g_dcb.Parity = NOPARITY;
		  g_dcb.StopBits = ONESTOPBIT;
		  if (SetCommState(g_hCom, &g_dcb) == 0)
			  return false;
		  return true;
	}
	
	bool Conexion::ConectaPuerto(CString puerto)
	{
		  //Creamos el puerto
		  g_hCom = CreateFile(puerto, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);

		  // Comprobamos resultado de apertura
		  if (g_hCom == INVALID_HANDLE_VALUE) 
			  return false;
	  
		  //PARAMETROS DEL PUERTO
		  if (GetCommState(g_hCom, &g_dcb)==0)
			  return false;
		  
		  //Establecemos velocidad a BAUDIOS
		  g_dcb.BaudRate = BAUDIOS;
		  g_dcb.fParity = FALSE;
		  g_dcb.fOutxCtsFlow = FALSE;
		  g_dcb.fOutxDsrFlow = FALSE;
		  g_dcb.fDtrControl = DTR_CONTROL_DISABLE;
		  g_dcb.fDsrSensitivity = FALSE;
		  g_dcb.fOutX = FALSE;
		  g_dcb.fInX = FALSE;
		  g_dcb.fRtsControl = RTS_CONTROL_DISABLE;
		  g_dcb.fAbortOnError = FALSE;
		  //Un byte
		  g_dcb.ByteSize = 8;
		  //Sin paridad
		  g_dcb.Parity = NOPARITY;
		  g_dcb.StopBits = ONESTOPBIT;
		  if (SetCommState(g_hCom, &g_dcb) == 0)
			  return false;
		  return true;
	}
	/**Funcin bool Conexion::DesconectaPuerto(). esta funcin desconecta el puerto de envio de comandos
	del robot.*/
	bool Conexion::DesconectaPuerto()
	{
		if (!CloseHandle(g_hCom) )
			return false;
		return true;
	}

	/**Funcin bool LeePuerto(int bytes). Esta funcin lee del puerto los bytes indicados. Por ejemplo 
	si llamamos a esta funcion de la siguiente forma LeePuerto(3), lo que hace es leer 3 bytes 
	del puerto y deja el resultado en la variable g_buffer.*/
	bool Conexion::LeePuerto(int bytes)
	{
	  if(!ReadFile(g_hCom, &g_buffer, bytes, &g_iBytes,NULL)) 
		return false;
	  return true;
	}

	/**Funcion bool Conexion::EscribePuerto(). Esta funcin escribe en el puerto los datos 
	que tenemos en buffer. */
	bool Conexion::EscribePuerto(int bytes)
	{		
	  if(!WriteFile(g_hCom, g_buffer, bytes, &g_iBytes,NULL)) 
		return false;
	  return true;
	}
	/**Funcion bool Conexion::ModoComando(Modo_Conexion mdCon). Esta funcin pone al robot a la espera
	de comandos. Adems indicamos la veloclidad de transmision (57600 baudios) por si se ha 
	desconfigurado. tambin indicamos el modo de conexin que puede ser seguro o completo. 
	En el modo seguro el robot funciona teniendo en cuenta la caida de las ruedas y el desnivel de stas.
	En el modo completo se tiene el todo el poder sobre el robot sin las restricciones del modo seguro*/
	bool Conexion::ModoComando(Modo_Conexion mdCon)
	{
		g_buffer[0] = 128;
		//apagado,pasivo,seguro,completo
		switch (mdCon) {
			case seguro:
				g_buffer[1] = 131;
				break;			
			case completo:
				g_buffer[1] = 132;
				break;			
			default: //No es un codigo valido
				return false;
		}		
		if (!EscribePuerto(2))
			return false;
		//A 57600 Baudios
		g_buffer[0] = 129;
		g_buffer[1] = 10;
		if (EscribePuerto(2) )
		{
			//Ademas inicializo los contadores de distancia y angulo
			short int nAux = 0;
			if (Entradas::Distancia(nAux) )
				return Entradas::Angulo(nAux);
		}
		return false;
	}

	/**Constructor Actuadores::Actuadores(). Esta clase es utilizada para el tratamiento de los
	actuadores del robot, como es la velocidad de las ruedas y el sonido a emitir*/
	Actuadores::Actuadores() {} 

	/**Destructor Actuadores::~Actuadores()*/
	Actuadores::~Actuadores() {} 
	
	/**Funcin bool Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en 
	mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
	Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
	bool Actuadores::Velocidad(int nVelocidad, int nRadio)
	{
		//Comprobamos que la velocidad est entre -500 mm y 500 mm
		if (nVelocidad < -500)
			nVelocidad = -500;
		if (nVelocidad > 500)
			nVelocidad = 500;
		//Comprobamos que el radio est entre -2000 y 2000 mm
		if (nRadio < -2000)
			nRadio = -2000;
		if (nRadio > 2000)
			nRadio = 2000;
		//[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]
		g_buffer[0] = 137; //Comando 137 es la velocidad
		g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
		g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
		g_buffer[3] = ( (nRadio & 0xFF00) >> 8); //Radio byte alto
		g_buffer[4] = (nRadio & 0xFF); //Radio byte bajo
		return Conexion::EscribePuerto(5);
	}
	
	/**Funcin bool Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en 
	mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
	Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
	bool Actuadores::Velocidad_Recto(int nVelocidad)
	{
		//Comprobamos que la velocidad est entre -500 mm y 500 mm
		if (nVelocidad < -500)
			nVelocidad = -500;
		if (nVelocidad > 500)
			nVelocidad = 500;		
		//[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]		
		g_buffer[0] = 137; //Comando 137 es la velocidad
		g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
		g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
		g_buffer[3] = 128;
		g_buffer[4] = 0;
		return Conexion::EscribePuerto(5);
	}

	/**Funcin bool Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en 
	mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
	Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
	bool Actuadores::Gira_Derecha(int nVelocidad)
	{
		//Comprobamos que la velocidad est entre -500 mm y 500 mm
		if (nVelocidad < -500)
			nVelocidad = -500;
		if (nVelocidad > 500)
			nVelocidad = 500;
				
		//[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]		
		g_buffer[0] = 137; //Comando 137 es la velocidad
		g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
		g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
		g_buffer[3] = 255;
		g_buffer[4] = 255;
		return Conexion::EscribePuerto(5);
	}

	/**Funcin bool Velocidad(int nVelocidad, int nRadio). Esta funcin establece la velocidad en 
	mm/s para las dos ruedas y el radio de giro. Si el radio es de 90 gira a la derecha.
	Si es de -90 gira a la izquierda y si es 0 va recto. Este radio se indica en mm */
	bool Actuadores::Gira_Izquierda(int nVelocidad)
	{
		//Comprobamos que la velocidad est entre -500 mm y 500 mm
		if (nVelocidad < -500)
			nVelocidad = -500;
		if (nVelocidad > 500)
			nVelocidad = 500;
		
		//[137] [Velocity high byte] [Velocity low byte][Radius high byte] [Radius low byte]		
		g_buffer[0] = 137; //Comando 137 es la velocidad
		g_buffer[1] = ( (nVelocidad & 0xFF00) >> 8); // velocidad byte alto
		g_buffer[2] = (nVelocidad & 0xFF); // Velocidad byte bajo
		g_buffer[3] = 0;
		g_buffer[4] = 1;
		return Conexion::EscribePuerto(5);
	}

	/**Funcin bool Velocidad_Independiente(int nRueda_Izq, int nRueda_Der). Con esta funcin indicamos
	la velocidad en mm/s para la rueda izquierda y la rueda derecha de forma independiente.*/
	bool Actuadores::Velocidad_Independiente(int nRueda_Izq, int nRueda_Der)
	{
		//Comprobamos que la velocidad est entre -500 mm y 500 mm
		if (nRueda_Izq < -500)
			nRueda_Izq = -500;
		if (nRueda_Izq > 500)
			nRueda_Izq = 500;
		
		if (nRueda_Der < -500)
			nRueda_Der = -500;
		if (nRueda_Der > 500)
			nRueda_Der = 500;
		
		//[145] [Right velocity high byte][Right velocity low byte] [Left velocity high byte][Left velocity low byte]
		g_buffer[0] = 145; //Comando 145 es la velocidad de cada rueda
		g_buffer[1] = ( (nRueda_Der & 0xFF00) >> 8); // velocidad der byte alto
		g_buffer[2] = (nRueda_Der & 0xFF); // Velocidad der byte bajo
		g_buffer[3] = ( (nRueda_Izq & 0xFF00) >> 8); //velocidad izq byte alto
		g_buffer[4] = (nRueda_Izq & 0xFF); //velocidad izq byte bajo
		return Conexion::EscribePuerto(5);
	}

	/**Funcin bool Actuadores::Salidas_Digitales(int nSalida1,int nSalida2,int nSalida3). Con esta 
	funcin ponemos las tres salidas digitales a nivel bajo o alto (0 o 1, respectivamente). 
	El 0 manda 0 voltios y el 1 logico envia 5 voltios. */
	bool Actuadores::Salidas_Digitales(int nSalida1,int nSalida2,int nSalida3)
	{
		int nValor = 0;

		if (nSalida1 != 0 && nSalida1 != 1)
			nSalida1 = 0;
		if (nSalida2 != 0 && nSalida2 != 1)
			nSalida2 = 0;
		if (nSalida3 != 0 && nSalida3 != 1)
			nSalida3 = 0;			
		//[147] [Output Bits]
		//Output bits = 0 0 0 0 0 nSalida3 nSalida2 nSalida 1 --> 1 BYTE
		//nSalida1 = Pin 19
		//nSalida2 = Pin 7
		//nSalida3 = Pin 20
		if (nSalida1 == 1)
			nValor = 1; //2 elevado a 0
		if (nSalida2 == 1)
			nValor += 2;//2 elevado a 1
		if (nSalida3 == 1)
			nValor += 4;//2 elevado a 2
		
		g_buffer[0] = 145; //Comando 145 es la velocidad de cada rueda
		g_buffer[1] = nValor; // Bits de salidas digitales
		return Conexion::EscribePuerto(2);
	}
	bool Actuadores::Script()
	{
		//152 17 137 1 44 128 0 156 1 144 137 1 44 0 1 157 0 90 153
		g_buffer[0] = 152;
		g_buffer[1] = 17;
		g_buffer[2] = 137;
		g_buffer[3] = 1;
		g_buffer[4] = 44;
		g_buffer[5] = 128;
		g_buffer[6] = 0;
		g_buffer[7] = 156;
		g_buffer[8] = 1;
		g_buffer[9] = 144;
		g_buffer[10] = 137;
		g_buffer[11] = 1;
		g_buffer[12] = 44;
		g_buffer[13] = 0;
		g_buffer[14] = 1;
		g_buffer[15] = 157;
		g_buffer[16] = 0;
		g_buffer[17] = 90;
		g_buffer[18] = 153;
		Conexion::EscribePuerto(19);
		Sleep(100);
		g_buffer[0] = 153;
		return Conexion::EscribePuerto(1);
	}
	/**Constructor Entradas::Entradas(). Esta clase es utilizada para la recepcin de
	sensores externos e internos del robot*/
	Entradas::Entradas() {} 

	/**Destructor Entradas::~Entradas()*/
	Entradas::~Entradas() {} 

	/**bool Entradas::Parachoque_Nivel_Ruedas(bool &bParachoque,bool &bRueda). Esta funcin nos
	devuelve si el parachoques del robot est chocando o alguna de las ruedas est en el aire (suspendida).
	El valor se devuelve en los argumentos pasados como referencia.*/
	bool Entradas::Parachoque_Nivel_Ruedas(bool &bParachoque,bool &bRueda)
	{
		
		char bit_0,bit_1,bit_2,bit_3,bit_4;
		int nObtenido;

		g_buffer[0] = 142; //Comando 142 pide dato del sensor indicado
		g_buffer[1] = 7; // Estado de las ruedas y parachoques
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Recojo la informacin, Es 1 byte 0 0 0 R_Castor R_Izq R_Der Golpe_Izq Golpe_Der
		if (!Conexion::LeePuerto(1) )
			return false;
		//En g_buffer tengo el byte de informacion. Extraigo los componentes
		nObtenido = g_buffer[0];
		bit_0 = (nObtenido & BITUNO);
		bit_1 = (nObtenido & BITDOS) >> 1;
		bit_2 = (nObtenido & BITTRES) >> 2;
		bit_3 = (nObtenido & BITCUATRO) >> 3;
		bit_4 = (nObtenido & BITCINCO) >> 4;
		//Bits 0 y 1 me dicen si hay golpe
		if (bit_0 == 1 || bit_1 == 1)
			bParachoque = true;
		else
			bParachoque = false;
		//Bits 2, 3 y 4 me dicen si hay caida de ruedas
		if (bit_2 == 1 || bit_3 == 1 || bit_4 == 1)
			bRueda = true;
		else
			bRueda = false;
		return true;
	}
	
	/**bool Entradas::Muro(bool &bMuro). Esta funcin devuelve si se detecta muro o no. Este valor
	se devuelve en el parmetro pasado por referencia.*/
	bool Entradas::Muro(bool &bMuro)
	{		
		int nObtenido;
		g_buffer[0] = 142; //Comando 142 pide dato del sensor indicado
		g_buffer[1] = 8; // Estado de las ruedas y parachoques
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Recojo la informacin, Es 1 byte 0 0 0 0 0 0 0 Muro
		if (!Conexion::LeePuerto(1) )
			return false;
		//En g_buffer tengo el byte de informacion. Extraigo los componentes
		nObtenido = g_buffer[0];
		if (nObtenido == 1)
			bMuro = true;
		else
			bMuro = false;
		return true;
	}

	/**Funcin bool Entradas::Desnivel(bool &bDesnivel). Esta funcin mira si hay algn desnivel en
	todo el conjunto de las ruedas. Para consultar las 4 posibilidades (desnivel izquierdo, 
	izquierdo-frontal, derecho y derecho-frontal) mandamos un paquete encapsulado de tal forma
	que en el mismo paquete obtenemos el estado de los 4 sensores*/
	bool Entradas::Desnivel(bool &bDesnivel)
	{		
		g_buffer[0] = 149; //Comando 149 pide dato de los sensores en forma lista
		g_buffer[1] = 4; //Cuantos sensores voy a pedir
		g_buffer[2] = 9;  // Desnivel izquierdo
		g_buffer[3] = 10; // Desnivel izquierdo-frontal
		g_buffer[4] = 11; // Desnivel derecho-frontal
		g_buffer[5] = 12; // Desnivel derecho
		if (!Conexion::EscribePuerto(6) ) 
			return false;
		//Obtendre 4 bytes. Del 9, 10, 11 y 12, ya que lo devuelve en el mismo orden
		//Para cada byte o devuelve 0 o 1
		if (!Conexion::LeePuerto(4) )
			return false;
		if (g_buffer[0] == 1 || g_buffer[1] == 1 || g_buffer[2] == 1 || g_buffer[3] == 1)
			bDesnivel = true;
		else
			bDesnivel = false;
		return true;
	}
	
	/**Funcin bool Entradas::Muro_Virtual(bool &bMuroV). Con esta funcin se detecta si existe o
	no el muro virtual, es decir si existe es que no puede pasar el robot.*/
	bool Entradas::Muro_Virtual(bool &bMuroV)
	{		
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 13;  //Sensor del muro virtual
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Obtendre 1 bytes que vale 0 1 si no se detecta o si el muro, respectivamente
		if (!Conexion::LeePuerto(1) )
			return false;
		if (g_buffer[0] == 1)
			bMuroV = true;
		else
			bMuroV = false;
		return true;
	}
	
	/**Funcin bool Entradas::Infrarojos(int &nEstado). Con esta funcin se el valor del 
	sensor infrarrojo del robot. Para todos los casos hay que comprar dispositivos adicionales
	para poder usar este mtodo. En cualquier caso en el parmetro nestado se pone el valor
	obtenido, cuyo valor se explica a continuacin:
	Remote Control:
		129 Left
		130 Forward
		131 Right
		132 Spot
		133 Max
		134 Small
		135 Medium
		136 Large / Clean
		137 PAUSE
		138 Power
		139 arc-forward-left
		140 arc-forward-right
		141 drive-stop.
	Scheduling Remote 
		142 Send All
		143 Seek Dock
	Home Base 
		240 Reserved
		248 Red Buoy
		244 Green Buoy
		242 Force Field
		252 Red Buoy and Green Buoy
		250 Red Buoy and Force Field
		246 Green Buoy and Force Field
		254 Red Buoy, Green Buoy and Force Field		
		255 No received byte. */
	bool Entradas::Infrarojos(short int &nEstado)
	{		
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 17;  //Sensor infrarrojo
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Obtendre 1 bytes que vale 0 1 si no se detecta o si el muro, respectivamente
		if (!Conexion::LeePuerto(1) )
			return false;
		nEstado = g_buffer[0];		
		return true;
	}

	/**Con esta funcin obtenemos el estado de los botones de play y avance. Si estan pulsados 
	en los parametros por referencia se devuelve true y sino false*/
	bool Entradas::Botones(bool &bPlay,bool &bAvance)
	{
		int bit_0,bit_2, nObtenido;

		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 18;  //Sensor botones
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Obtendre 1 bytes de la siguiente forma: 0 0 0 0 0 avance 0 play
		if (!Conexion::LeePuerto(1) )
			return false;
		nObtenido = g_buffer[0];
		bit_0 = (nObtenido & BITUNO);
		bit_2 = (nObtenido & BITTRES) >> 2;
		if (bit_0 == 1)
			bPlay = true;
		else
			bPlay = false;
		if (bit_2 == 1)
			bAvance = true;
		else
			bAvance = false;
		return true;
	}

	/**Esta funcin obitiene la distancia del robot en mm. Para ello se obtiene la distancia de las
	dos ruedas y se divide entre dos. Cada vez que se llama a esta funcin, ese contador se 
	incializa a 0 para la prxima vez que se llama. Si el valor es negativo significa que el 
	robot ha ido hacia atras*/
	bool Entradas::Distancia(short int &nDistancia)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 19;  //Sensor encoders de las ruedas
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nDistancia = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}

	/**Esta funcin obitiene el angulo de las ruedas del robot en grados. Cada vez que se llama 
	a esta funcin, ese contador se incializa a 0 para la prxima vez que se llama.*/
	bool Entradas::Angulo(short int &nAngulo)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 20;  //Sensor encoders de las ruedas para el angulo
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nAngulo = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene como se encuentra la bateria. Cargando, espera,...
	0 Not charging
	1 Reconditioning Charging
	2 Full Charging
	3 Trickle Charging
	4 Waiting
	5 Charging Fault Condition*/
	bool Entradas::Estado_Bateria(short int &nEstado)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 21;  //Estado de la bateria
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(1) )
			return false;		
		nEstado = g_buffer[0];
		return true;
	}
	/**Obtiene el voltaje de la bateria en mV*/
	bool Entradas::Voltaje_Bateria(short int &nVoltaje)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 22;  //Voltaje de la bateria
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nVoltaje = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la corriente de la bateria en mA*/
	bool Entradas::Corriente_Bateria(short int &nMiliamperios)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 23;  //Miliamperios de la bateria
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nMiliamperios = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la temperatura de la bateria en grados Celsius*/
	bool Entradas::Temperatura_Bateria(short int &nTemperatura)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 24;  //Temperatura de la bateria
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(1) )
			return false;		
		nTemperatura = g_buffer[0];
		return true;
	}
	/**Obtiene la carga que le queda a la bateria en mA hora*/
	bool Entradas::Carga_Bateria(short int &nCarga)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 25;  //Carga de la bateria que le queda
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nCarga = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la capacidad de la bateria en mA hora*/
	bool Entradas::Capacidad_Bateria(short int &nCapacidad)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 26;  //Capacidad de la bateria en mAHora
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nCapacidad = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de muro*/
	bool Entradas::Senal_Muro(short int &nSMuro)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 27;  //Intensidad de la seal del muro
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nSMuro = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de desnivel izquierdo*/
	bool Entradas::Senal_Desnivel_Izquierdo(short int &nSIzq)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 28;  //Intensidad del sensor de desnivel izquierdo
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nSIzq = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de desnivel izquierdo-frontal*/
	bool Entradas::Senal_Desnivel_Izquierdo_Fr(short int &nSIzq_Fr)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 29;  //Intensidad del sensor de desnivel izquierdo frontal
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nSIzq_Fr = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de desnivel derecho-frontal*/
	bool Entradas::Senal_Desnivel_Derecho_Fr(short int &nSDer_Fr)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 30;  //Intensidad del sensor de desnivel derecho frontal
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nSDer_Fr = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de desnivel derecho*/
	bool Entradas::Senal_Desnivel_Derecho(short int &nSDer)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 31;  //Intensidad del sensor de desnivel derecho
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		//Los 2 bytes vienen alreves
		nSDer = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene la intensidad del sensor de los 4 desniveles*/
	bool Entradas::Senal_Desniveles(short int &nSIzq,short int &nSIzq_Fr, short int &nSDer_Fr, short int &nSDer)
	{
		g_buffer[0] = 149; //Comando 149 pide dato de los sensores en forma lista
		g_buffer[1] = 4;   //Cuantos sensores voy a pedir
		g_buffer[2] = 28;  // intensidad Desnivel izquierdo
		g_buffer[3] = 29;  // intensidad Desnivel izquierdo-frontal
		g_buffer[4] = 30;  // intensidad Desnivel derecho-frontal
		g_buffer[5] = 31;  // intensidad Desnivel derecho
		if (!Conexion::EscribePuerto(6) ) 
			return false;
		if (!Conexion::LeePuerto(8) )
			return false;		
		nSIzq = g_buffer[1] + (g_buffer[0] << 8);
		nSIzq_Fr = g_buffer[3] + (g_buffer[2] << 8);
		nSDer_Fr = g_buffer[5] + (g_buffer[4] << 8);
		nSDer = g_buffer[7] + (g_buffer[6] << 8);
		return true;

	}
	/**Obtiene el valor de las 4 entradas digitales. esta entrada se corresponden
	con los pines 6,18, 5 y 17 para las entradas 3, 2, 1 y 0, respectivamente.*/
	bool Entradas::Entradas_Digitales(bool &bEntrada0, bool &bEntrada1, bool &bEntrada2, bool &bEntrada3)
	{
		int nEntrada0,nEntrada1,nEntrada2,nEntrada3,nObtenido;

		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 32;  //Entradas digitales
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Obtendre 1 bytes de la siguiente forma 0 0 0 baudios E3 E2 E1 E0
		if (!Conexion::LeePuerto(1) )
			return false;
		//Los 2 bytes vienen alreves
		nObtenido = g_buffer[0];
		nEntrada0 = (nObtenido & BITUNO);
		nEntrada1 = (nObtenido & BITDOS) >> 1;
		nEntrada2 = (nObtenido & BITTRES) >> 2;
		nEntrada3 = (nObtenido & BITCUATRO) >> 3;
		if (nEntrada0 == 1)
			bEntrada0 = true;
		else
			bEntrada0 = false;

		if (nEntrada1 == 1)
			bEntrada1 = true;
		else
			bEntrada1 = false;

		if (nEntrada2 == 1)
			bEntrada2 = true;
		else
			bEntrada2 = false;

		if (nEntrada3 == 1)
			bEntrada3 = true;
		else
			bEntrada3 = false;

		return true;
	}
	/**Obtiene el valor de la entradas analogica. El valor que obtenemos va
	de 0 a 1023 (0 = 0V y 1023 = 5V). El pin de la entrada es el 4*/
	bool Entradas::Entrada_Analogica(short int &nEntrada)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 33;  //Entrada analogica
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		nEntrada = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene las fuentes de carga disponibles*/
	bool Entradas::Fuente_Carga(bool &bHome,bool &bInterno)
	{
		int nHome,nInterno,nObtenido;

		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 34;  //Fuentes de carga disponible
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		//Obtendre 1 bytes de la siguiente forma 0 0 0 0 0 0 HOME INTERNO
		if (!Conexion::LeePuerto(1) )
			return false;
		nObtenido = g_buffer[0];
		nInterno = (nObtenido & BITUNO);
		nHome = (nObtenido & BITDOS) >> 1;
		if (nInterno == 1)
			bInterno = true;
		else
			bInterno = false;

		if (nHome == 1)
			bHome = true;
		else
			bHome = false;

		return true;
	}
	/**Obtiene el modo OI en el que esta trabajando:
	0 Off
	1 Passive
	2 Safe
	3 Full*/
	bool Entradas::Modo_OI(short int &nModo)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 35;  //Modo OI
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(1) )
			return false;		
		nModo = g_buffer[0];
		return true;
	}
	/**Obtiene el ultimo valor de velocidad que se pidio. En teoria debe de
	coincidir con la velocidad del robot*/
	bool Entradas::Velocidad_Solicitada(short int &nVelocidad)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 39;  //Velocidad del robot que se solicito
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		nVelocidad = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene el ultimo valor de radio solicitado. En teoria debe de coincidir 
	con el radio de las ruedas del robot*/
	bool Entradas::Radio_Solicitado(short int &nRadio)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 40;  //Radio de las ruedas
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		nRadio = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene el ultimo valor de velocidad de la rueda derecha que se pidio. */
	bool Entradas::Velocidad_Solic_Derecha(short int &nVelocidad_Der)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 41;  //Velocidad R derecha
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		nVelocidad_Der = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}
	/**Obtiene el ultimo valor de velocidad de la rueda izquierda que se pidio. */
	bool Entradas::Velocidad_Solic_Izquierda(short int &nVelocidad_Izq)
	{
		g_buffer[0] = 142; //Comando 142 pide dato del sensor
		g_buffer[1] = 42;  //Velocidad R izquierda
		if (!Conexion::EscribePuerto(2) ) 
			return false;
		if (!Conexion::LeePuerto(2) )
			return false;		
		nVelocidad_Izq = g_buffer[1] + (g_buffer[0] << 8);
		return true;
	}

	/**Con esta funcin espera el tiempo indicado en nTiempo en decimas de segundo. El tiempo va
	desde 0 a 255 decimas de segundo*/
	bool Esperar::Esperar_Tiempo(int nTiempo)
	{
		if (nTiempo < 0)
			nTiempo = 0;
		if (nTiempo > 255)
			nTiempo = 255;
			
		g_buffer[0] = 155; //Comando 155 esperar tiempo
		g_buffer[1] = nTiempo; //Tiempo indicado en decimas de segundo
		return Conexion::EscribePuerto(2);
	}
	/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
	la distancia indicada en mm. Es decir, hasta que no se recorra nDistancia mm el robot
	no ejecuta la siguiente instruccion. Adems, la variable interna de la distancia se 
	inicializa a 0. La distancia puede ser desde -32767 mm a 32768 mm*/
	bool Esperar::Esperar_Distancia(int nDistancia)
	{
		if (nDistancia < -32767)
			nDistancia = -32767;
		if (nDistancia > 32768)
			nDistancia = 32768;

		g_buffer[0] = 156; //Comando 156 esperar distancia
		g_buffer[1] = ( (nDistancia & 0xFF00) >> 8); // Distancia byte alto
		g_buffer[2] =   (nDistancia & 0xFF); // Distancia byte bajo
		return Conexion::EscribePuerto(3);
	}
	/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
	el angulo indicado en grados. Adems, la variable interna del ngulo se inicializa a 0. El valor
	del angulo va desde -32767 a 32768 grados.*/
	bool Esperar::Esperar_Angulo(int nAngulo)
	{
		if (nAngulo < -32767)
			nAngulo = -32767;
		if (nAngulo > 32768)
			nAngulo = 32768;

		g_buffer[0] = 157; //Comando 157 esperar angulo
		g_buffer[1] = ( (nAngulo & 0xFF00) >> 8); // Angulo byte alto
		g_buffer[2] =   (nAngulo & 0xFF); // Angulo byte bajo
		return Conexion::EscribePuerto(3);
	}
	/**Con esta funcin esperamos a que ocurra el evento que se especifica en el parametro.
	Este parmetro es del tipo Evento y segn el nombre de la enumeracin ya indica en si
	que evento es el que se espera*/
	bool Esperar::Esperar_Evento(Evento evento)
	{
		g_buffer[0] = 158; //Comando 158 esperar evento
		g_buffer[1] = evento; // que tipo de evento es el que queremos esperar
		return Conexion::EscribePuerto(2);
	}
}