#include <windows.h>
#include <String.h>
#include <atlstr.h>

#define BAUDIOS 57600
#define TAM_BUFFER 50

#define BITCERO   0x00
#define BITUNO    0x01
#define BITDOS    0x02
#define BITTRES   0x04
#define BITCUATRO 0x08
#define BITCINCO  0x10
#define BITSEIS   0x20
#define BITSIETE  0x40

//#if defined (__cplusplus)
namespace IRCR
{
	/**Clase para conexion, desconexion,...*/
    class Conexion 
    {
		public:
			/**Definicion de estructuras y variables*/
			__declspec(dllexport) enum Modo_Conexion {seguro,completo};			
			/**Funciones*/
			/**Constructor Conexion*/
			__declspec(dllexport) Conexion();
			/**Destructor Conexion*/
			__declspec(dllexport) ~Conexion();
			/**Conecta con el puerto serie especificado*/
			static __declspec(dllexport) bool ConectaPuerto(char *puerto);
			/**Tambien para conectar pero recibe un string*/
			static __declspec(dllexport) bool ConectaPuerto(CString puerto);
			/**Desconecta el puerto*/
			static __declspec(dllexport) bool DesconectaPuerto();					
			/**Lee del puerto abierto tantos bytes como se indiquen y los deja en g_buffer*/
			static __declspec(dllexport) bool LeePuerto(int bytes);
			/**Escribe en el puerto indicado el contenido de g_buffer*/
			static __declspec(dllexport) bool EscribePuerto(int bytes);
			/**Pone al robot en modo OI para poder enviar y recibir informacion por el puerto serie*/
			static __declspec(dllexport) bool ModoComando(Modo_Conexion mdCon);			
	};
	/**Clase para el tratamiento de los comandos de actuadores */
	class Actuadores 
	{
		public:		
			/**Constructor Actuadores*/
			__declspec(dllexport) Actuadores();			
			/**Destructor Actuadores*/
			__declspec(dllexport) ~Actuadores();
			/**Pone las dos ruedas del robot a girar a la velocidad indicada y con el ngulo de giro indicado*/
			static __declspec(dllexport) bool Velocidad(int nVelocidad, int nRadio);
			/**Velocidad para que vaya recto*/
			static __declspec(dllexport) bool Velocidad_Recto(int nVelocidad);
			/**Velocidad para que gire sentido horario*/
			static __declspec(dllexport) bool Gira_Derecha(int nVelocidad);
			/**Velocidad para que gire sentido antihorario*/
			static __declspec(dllexport) bool Gira_Izquierda(int nVelocidad);
			/**Cambia la velocidad de las ruedas de forma independiente*/
			static __declspec(dllexport) bool Velocidad_Independiente(int nRueda_Izq, int nRueda_Der);
			/**Cambia el valor de las tres salidas digitales del robot (pines 19, 7 y 20 del cargo bay conector*/
			static __declspec(dllexport) bool Salidas_Digitales(int nSalida1, int nSalida2, int nSalida3);			
			static __declspec(dllexport) bool Script();			
		
	};
	/**Clase para el tratamiento de las entradas (sensores, encoders, entradas adicionales,...)*/
	class Entradas
	{
		public:
			/**Constructor de Entradas*/
			__declspec(dllexport) Entradas();
			/**Destructor de Entradas*/
			__declspec(dllexport) ~Entradas();
			/**Obtiene el estado del parachoques y de la caida de las 3 ruedas*/
			static __declspec(dllexport) bool Parachoque_Nivel_Ruedas(bool &bParachoque,bool &bRueda);
			/**Obtiene el estado de muro para saber si hay obstaculo o no*/
			static __declspec(dllexport) bool Muro(bool &bMuro);
			/**Obtiene si alguna de las ruedas esta desniveladas, que son desnivel izquierdo,
			izquierdo frontal, derecho y derecho-frontal*/
			static __declspec(dllexport) bool Desnivel(bool &bDesnivel);
			/**Obtiene si hay choque con el muro virtual. De momento en desuso*/
			static __declspec(dllexport) bool Muro_Virtual(bool &bMuroV);
			/**Obtiene el valor del infrarojo. De momento en desuso*/
			static __declspec(dllexport) bool Infrarojos(short int &nEstado);
			/**Obtiene el estado de los botones de play y avance*/
			static __declspec(dllexport) bool Botones(bool &bPlay,bool &bAvance);
			/**Obtiene la distancia en mm que ha recorrido el robot. Cada vez que se llama a esta
			funcin se inicializa el contador a 0. Es decir, si la llamo una vez y me devuelve
			5 mm, ese miusmo contador se vuelve a inicializar a 0 desde ese momento.*/
			static __declspec(dllexport) bool Distancia(short int &nDistancia);
			/**Obtiene el ngulo engrados de las ruedas del robot. Cada vez que se llama a esta
			funcin se inicializa el contador a 0.*/
			static __declspec(dllexport) bool Angulo(short int &nAngulo);
			/**Obtiene como se encuentra la bateria. Cargando, espera,...*/
			static __declspec(dllexport) bool Estado_Bateria(short int &nEstado);
			/**Obtiene el voltaje de la bateria en mV*/
			static __declspec(dllexport) bool Voltaje_Bateria(short int &nVoltaje);
			/**Obtiene la corriente de la bateria en mA*/
			static __declspec(dllexport) bool Corriente_Bateria(short int &nMiliamperios);
			/**Obtiene la temperatura de la bateria en grados Celsius*/
			static __declspec(dllexport) bool Temperatura_Bateria(short int &nTemperatura);
			/**Obtiene la carga que le queda a la bateria en mA hora*/
			static __declspec(dllexport) bool Carga_Bateria(short int &nCarga);
			/**Obtiene la capacidad de la bateria en mA hora*/
			static __declspec(dllexport) bool Capacidad_Bateria(short int &nCapacidad);
			/**Obtiene la intensidad del sensor de muro*/
			static __declspec(dllexport) bool Senal_Muro(short int &nSMuro);
			/**Obtiene la intensidad del sensor de desnivel izquierdo*/
			static __declspec(dllexport) bool Senal_Desnivel_Izquierdo(short int &nSIzq);
			/**Obtiene la intensidad del sensor de desnivel izquierdo-frontal*/
			static __declspec(dllexport) bool Senal_Desnivel_Izquierdo_Fr(short int &nSIzq_Fr);
			/**Obtiene la intensidad del sensor de desnivel derecho-frontal*/
			static __declspec(dllexport) bool Senal_Desnivel_Derecho_Fr(short int &nSDer_Fr);
			/**Obtiene la intensidad del sensor de desnivel derecho*/
			static __declspec(dllexport) bool Senal_Desnivel_Derecho(short int &nSDer);
			/**Obtiene la intensidad del sensor de los 4 desniveles*/
			static __declspec(dllexport) bool Senal_Desniveles(short int &nSIzq,short int &nSIzq_Fr,short int &nSDer_Fr,short int &nSDer);
			/**Obtiene el valor de las 4 entradas digitales*/
			static __declspec(dllexport) bool Entradas_Digitales(bool &bEntrada0, bool &bEntrada1, bool &bEntrada2, bool &bEntrada3);
			/**Obtiene el valor de la entradas analogica*/
			static __declspec(dllexport) bool Entrada_Analogica(short int &nEntrada);
			/**Obtiene las fuentes de carga disponibles*/
			static __declspec(dllexport) bool Fuente_Carga(bool &bHome,bool &bInterno);
			/**Obtiene el modo OI en el que esta trabajando */
			static __declspec(dllexport) bool Modo_OI(short int &nModo);
			/**Obtiene el ultimo valor de velocidad que se pidio. En teoria debe de
			coincidir con la velocidad del robot*/
			static __declspec(dllexport) bool Velocidad_Solicitada(short int &nVelocidad);
			/**Obtiene el ultimo valor de radio solicitado. En teoria debe de coincidir 
			con el radio de las ruedas del robot*/
			static __declspec(dllexport) bool Radio_Solicitado(short int &nRadio);
			/**Obtiene el ultimo valor de velocidad de la rueda derecha que se pidio. */
			static __declspec(dllexport) bool Velocidad_Solic_Derecha(short int &nVelocidad_Der);
			/**Obtiene el ultimo valor de velocidad de la rueda izquierda que se pidio. */
			static __declspec(dllexport) bool Velocidad_Solic_Izquierda(short int &nVelocidad_Izq);
	};
	/**Clase utilizada para los eventos de espera*/
	class Esperar
	{
		public:
			/**Definicion de estructura para el evento de espera*/
			__declspec(dllexport) enum Evento {Rueda_Caida = 1,Rueda_Caida_FR = 2,Rueda_Caida_Izq = 3,
			Rueda_Caida_Der = 4,Parachoque = 5,Parachoque_Izq = 6,Parachoque_Der = 7,Muro_Virtual = 8,
			Muro = 9,Desnivel = 10,Desnivel_Izq = 11,Desnivel_Izq_Fr = 12,Desnivel_Der_Fr = 13,
			Desnivel_Der = 14, Home_Base = 15, Boton_Avance = 16, Boton_Play = 17, Entrada_Digital_0 = 18,
			Entrada_Digital_1 = 19, Entrada_Digital_2 = 20, Entrada_Digital_3 = 21, Modo_OI_Pasivo = 22,
			NO_Rueda_Caida = 255,NO_Rueda_Caida_FR = 254,NO_Rueda_Caida_Izq = 253,
			NO_Rueda_Caida_Der = 252,NO_Parachoque = 251,NO_Parachoque_Izq = 250,NO_Parachoque_Der = 249,NO_Muro_Virtual = 248,
			NO_Muro = 247,NO_Desnivel = 246,NO_Desnivel_Izq = 245,NO_Desnivel_Izq_Fr = 244,NO_Desnivel_Der_Fr = 243,
			NO_Desnivel_Der = 242, NO_Home_Base = 241, NO_Boton_Avance = 240, NO_Boton_Play = 239, NO_Entrada_Digital_0 = 238,
			NO_Entrada_Digital_1 = 237, NO_Entrada_Digital_2 = 236, NO_Entrada_Digital_3 = 235, NO_Modo_OI_Pasivo = 234};			

			/**Con esta funcin espera el tiempo indicado en nTiempo en decimas de segundo*/
			static __declspec(dllexport) bool Esperar_Tiempo(int nTiempo);
			/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
			la distancia indicada en mm. Es decir, hasta que no se recorra nDistancia mm el robot
			no ejecuta la siguiente instruccion.*/
			static __declspec(dllexport) bool Esperar_Distancia(int nDistancia);
			/**Con esta funcin se espera a ejecutar la siguiente instruccin hasta que recorra
			el angulo indicado en grados.*/
			static __declspec(dllexport) bool Esperar_Angulo(int nAngulo);
			/**Con esta funcin esperamos a que ocurra el evento que se especifica en el parametro.
			Este parmetro es del tipo Evento y segn el nombre de la enumeracin ya indica en si
			que evento es el que se espera*/
			static __declspec(dllexport) bool Esperar_Evento(Evento evento);	
	};
}
//#endif