#include "stdafx.h"
#include "Globales.h"
#include <Bt_api.h>

#define BAUDIOS 57600
#define TAM_BUFFER 50

DCB g_dcb; /**Para crear el puerto*/		
DWORD g_iBytes; /**Para saber los bytes que escribo o leo del puerto*/
HANDLE g_hCom; /**Referencia al puerto creado*/
unsigned char g_buffer[TAM_BUFFER]; /**Donde almacenamos los bytes leidos o que se van a escribir, del puerto*/
HANDLE g_ManejadorLib;

/**Funcion bool Conexion::ConectaPuerto(int puerto). Esta funcin configura todos los 
parametros del puerto "puerto", que es donde se encuentra ubicado el puerto serie del 
Robot. Estos parametros son los Baudios de velocidad, bit de paridad,... Luego, mediante 
el comando CreateFile creamos el puerto, como si de un fichero se tratara. Todo lo que 
leamos o escribamos en el puerto se quedara almacenado en la variable g_buffer. Adems, 
la conexin bluetooth se realiza de forma directa en este punto con la ayuda de RegisterDevice
y el identificador nico de la antena del robot.*/
bool ConectaPuerto(int puerto)
{
  //CONECTO CON EL DIPOSITIVO QUE YO QUIERO
  PORTEMUPortParams pp;
  BT_ADDR bRobot = 43925817773; //Identificador del robot Bluetooth
  memset (&pp, 0, sizeof(pp));
  pp.device = bRobot;  
  g_ManejadorLib = RegisterDevice (L"COM", puerto, L"btd.dll", (DWORD)&pp);

  WCHAR szComPort[30];
  wsprintf (szComPort, L"COM%d:", puerto);
  g_hCom = CreateFile(szComPort, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);

  // Comprobamos resultado de apertura
  if (g_hCom == INVALID_HANDLE_VALUE) 
	  return false;

  //PARAMETROS DEL PUERTO
  if (GetCommState(g_hCom, &g_dcb)==0)
	  return false;
  
  //Establecemos velocidad a BAUDIOS
  g_dcb.BaudRate = BAUDIOS;
  g_dcb.fParity = FALSE;
  g_dcb.fOutxCtsFlow = FALSE;
  g_dcb.fOutxDsrFlow = FALSE;
  g_dcb.fDtrControl = DTR_CONTROL_DISABLE;
  g_dcb.fDsrSensitivity = FALSE;
  g_dcb.fOutX = FALSE;
  g_dcb.fInX = FALSE;
  g_dcb.fRtsControl = RTS_CONTROL_DISABLE;
  g_dcb.fAbortOnError = FALSE;
  g_dcb.ByteSize = 8;
  g_dcb.Parity = NOPARITY;
  g_dcb.StopBits = ONESTOPBIT;
  if (SetCommState(g_hCom, &g_dcb) == 0)
	  return false;
  return true;	
}

/**Funcin bool Conexion::DesconectaPuerto(). esta funcin desconecta el puerto de envio de comandos
del robot.*/
bool DesconectaPuerto()
{
	DeregisterDevice (g_ManejadorLib);

	if (!CloseHandle(g_hCom) )
		return false;
	return true;
}

/**Funcin bool LeePuerto(int bytes). Esta funcin lee del puerto los bytes indicados. Por ejemplo 
si llamamos a esta funcion de la siguiente forma LeePuerto(3), lo que hace es leer 3 bytes 
del puerto y deja el resultado en la variable g_buffer.*/
bool LeePuerto(int bytes)
{
  if(!ReadFile(g_hCom, &g_buffer, bytes, &g_iBytes,NULL)) 
	return false;
  return true;
}

/**Funcion bool Conexion::EscribePuerto(). Esta funcin escribe en el puerto los datos 
que tenemos en buffer. */
bool EscribePuerto(int bytes)
{		
  if(!WriteFile(g_hCom, g_buffer, bytes, &g_iBytes,NULL)) 
	return false;
  return true;
}
/**Funcion bool Conexion::ModoComando(Modo_Conexion mdCon). Esta funcin pone al robot a la espera
de comandos. Adems indicamos la veloclidad de transmision (57600 baudios) por si se ha 
desconfigurado. tambin indicamos el modo de conexin que puede ser seguro o completo. 
En el modo seguro el robot funciona teniendo en cuenta la caida de las ruedas y el desnivel de stas.
En el modo completo se tiene el todo el poder sobre el robot sin las restricciones del modo seguro*/
bool ModoComando(Modo_Conexion mdCon)
{
	g_buffer[0] = 128;
	//apagado,pasivo,seguro,completo
	switch (mdCon) {
		case seguro:
			g_buffer[1] = 131;
			break;			
		case completo:
			g_buffer[1] = 132;
			break;			
		default: //No es un codigo valido
			return false;
	}		
	if (!EscribePuerto(2))
		return false;
	//A 57600 Baudios
	g_buffer[0] = 129;
	g_buffer[1] = 10;
	return EscribePuerto(2);
}

/**Funcin bool Velocidad_Independiente(int nRueda_Izq, int nRueda_Der). Con esta funcin indicamos
la velocidad en mm/s para la rueda izquierda y la rueda derecha de forma independiente.*/
bool Velocidad_Independiente(int nRueda_Izq, int nRueda_Der)
{
	//Comprobamos que la velocidad est entre -500 mm y 500 mm
	if (nRueda_Izq < -500)
		nRueda_Izq = -500;
	if (nRueda_Izq > 500)
		nRueda_Izq = 500;
	
	if (nRueda_Der < -500)
		nRueda_Der = -500;
	if (nRueda_Der > 500)
		nRueda_Der = 500;
	
	//[145] [Right velocity high byte][Right velocity low byte] [Left velocity high byte][Left velocity low byte]
	g_buffer[0] = 145; //Comando 145 es la velocidad de cada rueda
	g_buffer[1] = ( (nRueda_Der & 0xFF00) >> 8); // velocidad der byte alto
	g_buffer[2] = (nRueda_Der & 0xFF); // Velocidad der byte bajo
	g_buffer[3] = ( (nRueda_Izq & 0xFF00) >> 8); //velocidad izq byte alto
	g_buffer[4] = (nRueda_Izq & 0xFF); //velocidad izq byte bajo
	return EscribePuerto(5);
}
