// MandoDlg.cpp: archivo de implementacin
//

#include "stdafx.h"
#include "Mando.h"
#include "MandoDlg.h"
#include "Globales.h"
#include <smiAccelerometer.h>

#define PUERTO 6

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// Cuadro de dilogo de CMandoDlg
#define ACELERACION 350 //Aceleracion del robot para ir hacia delante y atras
#define RADIO       200 //Aceleracion de curvatura del robot

CMandoDlg::CMandoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMandoDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CMandoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CMandoDlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
	ON_WM_SIZE()
#endif
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BTNCONECTAR, &CMandoDlg::OnBnClickedBtnconectar)
	ON_BN_CLICKED(IDC_BUTTON1, &CMandoDlg::OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON2, &CMandoDlg::OnBnClickedButton2)
END_MESSAGE_MAP()


// Controladores de mensaje de CMandoDlg

BOOL CMandoDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Establecer el icono para este cuadro de dilogo. El marco de trabajo realiza esta operacin
	//  automticamente cuando la ventana principal de la aplicacin no es un cuadro de dilogo
	SetIcon(m_hIcon, TRUE);			// Establecer icono grande
	SetIcon(m_hIcon, FALSE);		// Establecer icono pequeo

	// TODO: agregar aqu inicializacin adicional
	g_bActivado=false;
	GetDlgItem(IDC_ICONO1)->ShowWindow(FALSE);
	GetDlgItem(IDC_ICONO2)->ShowWindow(TRUE);
	UpdateData(FALSE); //Actualizar variables miembro
	return TRUE;  // Devuelve TRUE  a menos que establezca el foco en un control
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CMandoDlg::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
	if (AfxIsDRAEnabled())
	{
		DRA::RelayoutDialog(
			AfxGetResourceHandle(), 
			this->m_hWnd, 
			DRA::GetDisplayMode() != DRA::Portrait ? 
			MAKEINTRESOURCE(IDD_MANDO_DIALOG_WIDE) : 
			MAKEINTRESOURCE(IDD_MANDO_DIALOG));
	}
}
#endif

/**OnBnClickedBtnconectar(). Cuando se pulsa sobre el botn conectar se llama a esta funcin.
En caso de querer conectarnos con el robot, primero se conecta de forma directa con la antena
bluetooth del robot y despus abre el puerto serie virtual para pooder leer y escribir del puerto.
Tambin ponemos el robot en modo comando, para que est preparado para enviar y recibir comandos.
en caso de desconectarnos, simplemente cerramos el puerto y paramos el robot.*/
void CMandoDlg::OnBnClickedBtnconectar()
{
	Modo_Conexion modo;
	modo = seguro;
	UpdateData(TRUE);
	//Abrimos o cerramos el puerto
	if (!g_bActivado) //Abrimos el puerto
	{	  
	  ConectaPuerto(PUERTO);
	  ModoComando(modo);
	  //Lo activamos
	  g_bActivado=true;
	  //Ponemos el boton verde
	  GetDlgItem(IDC_ICONO1)->ShowWindow(TRUE);
	  GetDlgItem(IDC_ICONO2)->ShowWindow(FALSE);	  
	} else //cerramos el puerto
	{
	  Velocidad_Independiente(0,0);
	  DesconectaPuerto();
	  g_bActivado=false;
	  //Ponemos el boton rojo
	  GetDlgItem(IDC_ICONO1)->ShowWindow(FALSE);
	  GetDlgItem(IDC_ICONO2)->ShowWindow(TRUE);
	}	
	UpdateData(FALSE);
}

void CMandoDlg::OnOK()
{
  if (g_bActivado)
	  DesconectaPuerto();
  CDialog::OnOK();
}

/**Esta funcin se llama cuando pulsamo sobre el botn COMENZAR. Lanza un hilo de ejecucin
que es el encargado de obtener los valores de los acelerometros y calcular las velocidades.*/
void CMandoDlg::OnBnClickedButton1()
{
	// TODO: Agregue aqu su cdigo de controlador de notificacin de control
	pThread = AfxBeginThread(ejecutarComandos, this, THREAD_PRIORITY_HIGHEST);
	if (pThread == NULL)
		MessageBox(_T("No se ha podido crear el hilo de ejecucin"),0,MB_ICONEXCLAMATION);
	else
	{
		GetDlgItem(IDC_BUTTON1)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON2)->EnableWindow(TRUE);
	}
}

/**Esta funcin es la que se lanza cuando creamos el hilo de ejecucin desde el botn COMENZAR. 
Es un bucle infinito que obtiene los valores de los acelermetros cada 50 ms y calcula las velocidades
de las ruedas izquireda y derecha que tenemos que introducir en el robot en funcin de los valores
de los acelermetros.*/
UINT ejecutarComandos( LPVOID pParam)
{
	SmiAccelerometerVector vector;
	CString aux;
	float flVel_Izq, flVel_Der;
	int nVel_Izq, nVel_Der;
	while (1)
	{
		SmiAccelerometerGetVector(&vector);
		//Calculamos velocidad rueda izquierda y derecha
		flVel_Izq = (ACELERACION * vector.z) + (RADIO * vector.y);
		flVel_Der = (ACELERACION * vector.z) - (RADIO * vector.y);
		nVel_Izq = (int)floor(flVel_Izq);
		nVel_Der = (int)floor(flVel_Der);
		Velocidad_Independiente(nVel_Izq,nVel_Der);
		aux.Format(_T("%f"),vector.y);
		((CMandoDlg *)pParam)->SetDlgItemText(IDC_EDITEJEY,aux);
		aux.Format(_T("%f"),vector.z);
		((CMandoDlg *)pParam)->SetDlgItemText(IDC_EDITEJEZ,aux);
		Sleep(50);
	}
	return 0;
}
/**esta funcin es llamada cuando pulsamos sobre el botn TERMINAR. Lo nico que hace es matar el hilo
de ejecucin del programa para as poder terminar con la aplicacin mando.exe.*/
void CMandoDlg::OnBnClickedButton2()
{
	// TODO: Agregue aqu su cdigo de controlador de notificacin de control
	//Matamos el hilo
	//Ponemos el boton desabilitado
	GetDlgItem(IDC_BUTTON1)->EnableWindow(TRUE);
	GetDlgItem(IDC_BUTTON2)->EnableWindow(FALSE);
	TerminateThread(pThread->m_hThread,0);
}
